# -*- coding: utf-8 -*-
import sys
import os
import pkg_resources

sys.path.insert(0, os.path.abspath('../examples'))

__distribution = pkg_resources.get_distribution('Sentaku')

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.viewcode',
    'sphinx.ext.todo',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'

# General information about the project.
project = __distribution.project_name
copyright = u'2016, Ronny Pfannschmidt'
author = u'Ronny Pfannschmidt'

release = __distribution.version
version = '.'.join(release.split('.')[:2])

exclude_patterns = []

pygments_style = 'sphinx'
todo_include_todos = False

html_theme = 'haiku'
html_static_path = ['_static']

htmlhelp_basename = 'sentakudoc'



"""

    utility classes for implementation lookup
"""
from collections import Mapping
import attr


@attr.s
class ImplementationName(object):
    """
    utility class to declare names for implementations
    """
    name = attr.ib()
    documentation = attr.ib(repr=False, default=None)


@attr.s
class AttributeBasedImplementations(Mapping):

    holder = attr.ib()
    attribute_mapping = attr.ib(validator=attr.validators.instance_of(Mapping))

    def __getitem__(self, key):
        attribute_name = self.attribute_mapping[key]
        result = getattr(self.holder, attribute_name, self)
        if result is self:
            raise LookupError('{holder!r} has no attribute {name}'.format(
                holder=self.holder,
                name=attribute_name,
            ))
        return result

    def __iter__(self):
        return iter(self.attribute_mapping)

    def __len__(self):
        return len(self.attribute_mapping)



from sentaku.context import ImplementationContext


def test_from_instances():
    ctx = ImplementationContext.from_instances([1, 'a'])
    assert ctx.implementation_chooser.current.elements == [int, str]
    assert ctx.implementations == {int: 1, str: 'a'}



import warnings

from .utils import (
    ImplementationName,
    AttributeBasedImplementations,
)

from .modeling import Element, Collection
from .context import (
    ContextualMethod, ContextualProperty,
    ImplementationContext,
)


def register_external_implementations_in(*modules):
    warnings.warn(
        message="register_external_implementations_in is deprecated,"
                " please use importscan.scan",
        category=DeprecationWarning,
        stacklevel=2,
    )
    import importscan
    for module in modules:
        importscan.scan(module)


__all__ = [
    'Element',
    'Collection',

    'ContextualMethod',
    'ContextualProperty',
    'ImplementationContext',

    'ImplementationName',
    'AttributeBasedImplementations',
]



#!/usr/bin/env python

from setuptools import setup

setup(name="sentaku",
      use_scm_version=True,
      author="RonnyPfannschmidt",
      author_email="opensource@ronnypfannschmidt.de",
      description="variadic ux implementation for testing",
      license="MPLv2",
      keywords=["testing"],
      url="https://github.com/RonnyPfannschmidt/Sentaku",
      packages=["sentaku"],
      package_dir={'': 'src'},
      setup_requires=[
          'setuptools_scm',
      ],
      install_requires=[
        'attrs >= 16.3.0',
        'reg >= 0.10',
        'dectate >= 0.12',
        'importscan',
      ],
      extras_require={
        'test': ['pytest'],
        'examples': ['selenium'],
      },
      classifiers=[
          "Topic :: Utilities",
          "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.4",
          "Programming Language :: Python :: 3.5",
          "Intended Audience :: Developers",
          "Development Status :: 4 - Beta",
      ])
